/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;

public class OneSampleTTest {

	public static double calculatePValue (double [] values, double testBase) throws ChipMonkException {

		if (values == null) {
			throw new ChipMonkException ("List of values for t-test cannot be null");
		}
		if (values.length < 2) {
			throw new ChipMonkException("At least two values are required for a t-test");
		}
		double mean =0;
		for (int i=0;i<values.length;i++){
			mean += values[i];
		}
		mean /= values.length;
		
		double stdev = 0;
		for (int i=0;i<values.length;i++) {
			stdev += Math.pow((values[i]-mean),2);
		}
		stdev /= (values.length-1);
		stdev = Math.sqrt(stdev);
		
		double sem = stdev / (Math.sqrt(values.length));
		
		double tValue = (mean-testBase)/sem;
		
		double pValue = TDistribution.tProb(values.length-1,tValue);
		
		return pValue;
	}
	
}
