/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataTypes;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class ProbeSet {
		
	private Hashtable probeList = new Hashtable((int)(ChipMonkPreferences.getInstance().getApproxProbeSetSize()*1.33f),0.75f);
	private Vector probeLists = new Vector();
	private ProbeList activeList = null;
	private Vector listeners = new Vector();
	
	
	public void addProbe (Probe p) {
		probeList.put(p.name(),p);
	}
	
	public Probe [] getAllProbes () {
		return (Probe [])probeList.values().toArray(new Probe [0]);	
	}
	
	public ProbeList getAllProbesList () {
		ProbeList p = new ProbeList(this,"All Probes",null);
		Enumeration e = probeList.elements();
		while (e.hasMoreElements()) {
			p.addProbe((Probe)e.nextElement(),null);
		}
		return p;
	}
		
	public boolean containsProbeWithName (String name) {
		if (probeList.containsKey(name)) {
			return true;
		}
		return false;
	}
		
	public Probe getProbeWithName (String name) {
		return (Probe)probeList.get(name);
	}
	
	public void setActiveList (ProbeList list) throws ChipMonkException {
		
		if (list == null) {
			activeList = null;
			return;
		}
		// We shouldn't be passed a list we don't know about
		// but it could happen.
		if (! probeLists.contains(list)) {
			addProbeList(list);
		}
		activeList = list;
	}
	
	public ProbeList getActiveProbeList () {
		if (activeList != null) {
			return activeList;
		}
		else {
			return getAllProbesList();
		}
	}
	
	public void addProbeList (ProbeList list) throws ChipMonkException {
		// We need to check that all the probes in the
		// list we've been passed are present in our
		// full list.
		Probe [] probes = list.getAllProbes();
		for (int i=0;i<probes.length;i++) {
			if (! this.containsProbeWithName(probes[i].name())){
				throw new ChipMonkException("Probe list "+list.getListName()+" contained probe "+probes[i].name()+" which isn't in the full probe set");
			}
		}
		probeLists.add(list);
		
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((ProbeSetChangeListener)e.nextElement()).probeListAdded(list);
		}

	}
	
	public void removeProbeList (ProbeList list) {
		probeLists.removeElement(list);
		
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((ProbeSetChangeListener)e.nextElement()).probeListRemoved(list);
		}

	}
	
	public String getActiveProbeListName () {
		if (activeList == null)
			return "All Probes";
		else
			return activeList.getListName();
	}
	
	public Probe [] getProbesForChromosome (Chromosome c) {
		Vector tempChr = new Vector();
		Iterator i = probeList.values().iterator();
		
		while (i.hasNext()) {
			Probe p = (Probe)i.next();
			if (p.chromosome() == c) {
				tempChr.add(p);
			}
		}
		Probe [] chrProbes = (Probe [])tempChr.toArray(new Probe [0]);
		
		Arrays.sort(chrProbes);
		
		return chrProbes;
	}
	
	public Probe [] getActiveProbes () {
		if (activeList != null) {
			return activeList.getAllProbes();
		}
		else {
			return getAllProbes();
		}
	}
	
	public Probe [] getActiveProbesForChromosome (Chromosome c) {
		Probe [] list;
		if (activeList != null) {
			list = activeList.getAllProbes();
		}
		else {
			list = getAllProbes();
		}
		Vector tempChr = new Vector();
			
		for (int i=0;i<list.length;i++) {
			if (list[i].chromosome() == c) {
				tempChr.add(list[i]);
			}
		}
		Probe [] chrProbes = (Probe [])tempChr.toArray(new Probe [0]);
			
		Arrays.sort(chrProbes);
			
		return chrProbes;
	}


	public ProbeList [] getAllProbeLists () {
		ProbeList [] lists = (ProbeList [])probeLists.toArray(new ProbeList[0]);
		Arrays.sort(lists);
		return lists;
	}
	
	public void addProbeSetChangeListener (ProbeSetChangeListener l) {
		listeners.add(l);
	}

	public void removeProbeSetChangeListener (ProbeSetChangeListener l) {
		listeners.remove(l);
	}
	
	public void probeListRenamed (ProbeList l) {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((ProbeSetChangeListener)e.nextElement()).probeListRenamed(l);
		}

	}

	
}
