/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class EditPreferencesDialog extends JDialog implements ActionListener,ListSelectionListener {

	private JTextField genomeBase;
	private JTextField dataLocation;
	private JTextField saveLocation;
	private JTextField probeSetSize;
	private JTextField proxyHost;
	private JTextField proxyPort;
	private JTextField downloadLocation;
	private JList loadedFeatures;
	private DefaultListModel loadedFeaturesModel;
	private JButton add;
	private JButton remove;
	private ChipMonkApplication application;
	
	public EditPreferencesDialog (ChipMonkApplication application) {
		super(application,"Edit Preferences...");
		setSize(460,230);
		setLocationRelativeTo(application);
		setModal(true);
		this.application = application;
		ChipMonkPreferences p = ChipMonkPreferences.getInstance();
		
		JTabbedPane tabs = new JTabbedPane();

		JPanel filePanel = new JPanel();
		filePanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.1;
		c.weighty=0.5;
		c.fill = GridBagConstraints.HORIZONTAL;
		filePanel.add(new JLabel("Genome Base Location"),c);
		c.gridx=1;
		c.weightx=0.5;
		genomeBase = new JTextField();
		try {
			genomeBase.setText(p.getGenomeBase().getAbsolutePath());
		}
		catch (FileNotFoundException e){
			JOptionPane.showMessageDialog(this, "Couldn't find the folder which was supposed to hold the genomes", "Warning", JOptionPane.WARNING_MESSAGE);
		}
		genomeBase.setEditable(false);
		filePanel.add(genomeBase,c);
		c.gridx=2;
		c.weightx=0.1;
		JButton genomeButton = new JButton("Browse");
		genomeButton.setActionCommand("genomeBase");
		genomeButton.addActionListener(this);
		filePanel.add(genomeButton,c);
		
		c.gridx=0;
		c.gridy++;
		c.weightx=0.1;
		filePanel.add(new JLabel("Default Data Location"),c);
		c.gridx=1;
		c.weightx=0.5;
		dataLocation = new JTextField(p.getDataLocation().getAbsolutePath());
		dataLocation.setEditable(false);
		filePanel.add(dataLocation,c);
		c.gridx=2;
		c.weightx=0.1;
		JButton dataButton = new JButton("Browse");
		dataButton.setActionCommand("dataLocation");
		dataButton.addActionListener(this);
		filePanel.add(dataButton,c);

		c.gridx=0;
		c.gridy++;
		c.weightx=0.1;
		filePanel.add(new JLabel("Default Save Location"),c);
		c.gridx=1;
		c.weightx=0.5;
		saveLocation = new JTextField(p.getSaveLocation().getAbsolutePath());
		saveLocation.setEditable(false);
		filePanel.add(saveLocation,c);
		c.gridx=2;
		c.weightx=0.1;
		JButton saveLocationButton = new JButton("Browse");
		saveLocationButton.setActionCommand("saveLocation");
		saveLocationButton.addActionListener(this);
		filePanel.add(saveLocationButton,c);

		tabs.addTab("Files",filePanel);
		
		JPanel memoryPanel = new JPanel();
		
		memoryPanel.setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.1;
		c.weighty=0.5;
		c.fill = GridBagConstraints.HORIZONTAL;
		
		memoryPanel.add(new JLabel("Approx probeset size"),c);
		c.gridx=1;
		c.weightx=0.5;
		probeSetSize = new JTextField(""+p.getApproxProbeSetSize());
		memoryPanel.add(probeSetSize,c);
		
		c.gridx=0;
		c.gridy++;
		memoryPanel.add(new JLabel("Load only feature types"),c);
		
		c.gridx=1;
		JPanel featureTypesPanel = new JPanel();
		featureTypesPanel.setLayout(new BorderLayout());
		loadedFeaturesModel = new DefaultListModel();
		String [] f = p.getLoadedFeatures();
		for (int i=0;i<f.length;i++) {
//			System.out.println("Added "+f[i]);
			loadedFeaturesModel.addElement(f[i]);
		}
		loadedFeatures = new JList(loadedFeaturesModel);
		loadedFeatures.addListSelectionListener(this);
		featureTypesPanel.add(new JScrollPane(loadedFeatures),BorderLayout.CENTER);
		JPanel featureTypesButtons = new JPanel();
		featureTypesButtons.setLayout(new GridLayout(2,1));
		add = new JButton("Add");
		add.setActionCommand("addFeature");
		add.addActionListener(this);
		featureTypesButtons.add(add);
		remove = new JButton("Remove");
		remove.setActionCommand("removeFeature");
		remove.addActionListener(this);
		remove.setEnabled(false);
		featureTypesButtons.add(remove);
		featureTypesPanel.add(featureTypesButtons,BorderLayout.EAST);
		
		
		memoryPanel.add(featureTypesPanel,c);
		
		c.gridx=0;
		c.gridy++;
		c.gridwidth=2;
//		JTextArea startUpMemory = new JTextArea("ChipMonk is VERY memory intensive when working with large datasets or heavily annotated genomes.  The amount of memory the program can use is set in the java startup command using the parameters -Xms and -Xmx for the initial and maximum memory allocation.  Increasing these values will make the program run more smoothly.  On windows these values are set in the \"run_chipmonk.bat\" file.");
		JTextArea startUpMemory = new JTextArea("See also the help section on memory usage.");
		startUpMemory.setWrapStyleWord(true);
		startUpMemory.setLineWrap(true);
		memoryPanel.add(startUpMemory,c);
		startUpMemory.setBackground(memoryPanel.getBackground());
		tabs.addTab("Memory", memoryPanel);
		
		
		JPanel networkPanel = new JPanel();
		
		networkPanel.setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.1;
		c.weighty=0.5;
		c.fill = GridBagConstraints.HORIZONTAL;
		
		networkPanel.add(new JLabel("HTTP Proxy server"),c);
		c.gridx=1;
		c.weightx=0.5;
		proxyHost = new JTextField(p.proxyHost());
		networkPanel.add(proxyHost,c);

		c.gridx=0;
		c.gridy++;
		c.weightx=0.1;
		networkPanel.add(new JLabel("HTTP Proxy port"),c);
		c.gridx=1;
		c.weightx=0.5;
		proxyPort = new JTextField(""+p.proxyPort());
		networkPanel.add(proxyPort,c);
		
		c.gridx=0;
		c.gridy++;
		c.weightx=0.1;
		networkPanel.add(new JLabel("Genome Download URL"),c);
		c.gridx=1;
		c.weightx=0.5;
		downloadLocation = new JTextField(p.getGenomeDownloadLocation());
		networkPanel.add(downloadLocation,c);
		tabs.addTab("Network",networkPanel);
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(tabs, BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		JButton okButton = new JButton("OK");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	private void getDir (JTextField f) {
		JFileChooser chooser = new JFileChooser(); 
	    chooser.setCurrentDirectory(new File(f.getText()));
	    chooser.setDialogTitle("Select Directory");
	    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
	    	f.setText(chooser.getSelectedFile().getAbsolutePath());
	    }
	}
	
	public void actionPerformed(ActionEvent ae) {
		String c = ae.getActionCommand();
		
		if (c.equals("genomeBase")) {
			getDir(genomeBase);
		}
		else if (c.equals("dataLocation")) {
			getDir(dataLocation);
		}
		else if (c.equals("saveLocation")) {
			getDir(saveLocation);
		}
		else if (c.equals("removeFeature")) {
			Object [] o = loadedFeatures.getSelectedValues();
			for (int i=0;i<o.length;i++) {
				loadedFeaturesModel.removeElement(o[i]);
			}
		}
		else if (c.equals("addFeature")) {
			String featureName=null;
			UnloadedFeatureSelector ufs = new UnloadedFeatureSelector(this);
			while (true) {
				featureName = ufs.getFeatureName();
				if (featureName == null)
					return;  // They cancelled
					
					
				if (featureName.length() == 0)
					continue; // Try again
				
				break;
			}
			ufs.dispose();
			loadedFeaturesModel.addElement(featureName);
		}
		else if (c.equals("cancel")) {
			setVisible(false);
			dispose();
		}
		
		else if (c.equals("ok")) {
			File genomeBaseFile = new File(genomeBase.getText());
			if (genomeBaseFile == null || (! genomeBaseFile.exists())) {
				JOptionPane.showMessageDialog(this,"Invalid genome base location","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}

			File dataLocationFile = new File(dataLocation.getText());
			if (dataLocationFile == null || (! dataLocationFile.exists())) {
				JOptionPane.showMessageDialog(this,"Invalid data location","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			
			File saveLocationFile = new File(saveLocation.getText());
			if (saveLocationFile == null || (! saveLocationFile.exists())) {
				JOptionPane.showMessageDialog(this,"Invalid save location","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			
			int probeSetSizeValue=0;
			try {
				probeSetSizeValue = Integer.parseInt(probeSetSize.getText());
			}
			catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(this,"Probe set size was not an integer","Error",JOptionPane.ERROR_MESSAGE);
				return;				
			}

			String proxyHostValue = proxyHost.getText();
			int proxyPortValue = 0;
			if (proxyPort.getText().length()>0) {
				try {
					proxyPortValue = Integer.parseInt(proxyPort.getText());
				}
				catch (NumberFormatException e) {
					JOptionPane.showMessageDialog(this,"Proxy port number was not an integer","Error",JOptionPane.ERROR_MESSAGE);
					return;				
				}
			}
			
			if (proxyHostValue.length()>0 && proxyPort.getText().length() == 0){
				JOptionPane.showMessageDialog(this,"You specified a proxy server address, but did not provide the port number (default is usually 80 or 8080)","Error",JOptionPane.ERROR_MESSAGE);
				return;								
			}
			
			
			// OK that's everything which could have gone wrong.  Let's save it
			// to the preferences file
			
			ChipMonkPreferences p = ChipMonkPreferences.getInstance();
			
			p.setApproxProbeSetSize(probeSetSizeValue);
			p.setDataLocation(dataLocationFile);
			p.setSaveLocation(saveLocationFile);
			p.setGenomeBase(genomeBaseFile);
			p.setProxy(proxyHostValue,proxyPortValue);
			p.setGenomeDownloadLocation(downloadLocation.getText());
			Object [] o = loadedFeaturesModel.toArray();
			String [] s = new String[o.length];
			for (int i=0;i<s.length;i++) {
				s[i] = (String)o[i];
			}
			p.setDefaultAnnotations(s);
			
			try {
				p.savePreferences();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(this,"Error saving preferences: "+e.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			setVisible(false);
		}
	}

	public void valueChanged(ListSelectionEvent arg0) {
		if (loadedFeatures.getSelectedIndices().length>0) {
			remove.setEnabled(true);
		}
		else {
			remove.setEnabled(false);
		}
	}
	
	private class UnloadedFeatureSelector extends JDialog implements ListSelectionListener, ActionListener {
		private boolean cancelled = true;
		private JTextField input;
		private JList list;
		
		public UnloadedFeatureSelector (Dialog c) {
			super(c);
			setTitle("Select feature name");
			setSize(250,250);
			setModal(true);
			setLocationRelativeTo(c);
			
			JPanel topPanel = new JPanel();
			topPanel.setLayout(new BorderLayout());
			input = new JTextField("[Enter feature name]");
			topPanel.add(input,BorderLayout.CENTER);
			topPanel.add(new JLabel("Currently unloaded features"),BorderLayout.SOUTH);
			getContentPane().setLayout(new BorderLayout());
			getContentPane().add(topPanel,BorderLayout.NORTH);

			if (application.genome() != null) {
				list = new JList (application.genome().listUnloadedFeatureTypes());
			}
			else {
				list = new JList ();
			}
			list.addListSelectionListener(this);
			getContentPane().add(new JScrollPane(list),BorderLayout.CENTER);
			
			JPanel buttonPanel = new JPanel();

			JButton cancelButton = new JButton("Cancel");
			cancelButton.setActionCommand("cancel");
			cancelButton.addActionListener(this);
			buttonPanel.add(cancelButton);
			
			JButton okButton = new JButton("OK");
			okButton.setActionCommand("ok");
			okButton.addActionListener(this);
			buttonPanel.add(okButton);
			
			getContentPane().add(buttonPanel,BorderLayout.SOUTH);
			
			
		}
		
		public String getFeatureName (){
			cancelled = true;
			input.setText("[Enter feature name]");
			setVisible(true);
			if (cancelled) {
				return null;
			}
			else {
				return input.getText();
			}
		}

		public void valueChanged(ListSelectionEvent e) {
			String s = (String)list.getSelectedValue();
			if (s!=null) {
				input.setText(s);
			}
		}

		public void actionPerformed(ActionEvent e) {
			if (e.getActionCommand().equals("ok")) {
				cancelled = false;
			}
			if (input.getText().equals("[Enter feature name]")){
				input.setText("");
			}
			
			setVisible(false);
		}
	}
	
}
