/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Displays.DataViewer;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataChangeListener;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataCollection;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataGroup;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeSetChangeListener;

public class DataViewer extends JPanel  implements MouseListener, TreeSelectionListener, DataChangeListener, ProbeSetChangeListener {

	private DataCollection collection;
	private ChipMonkApplication application;

	private DefaultTreeModel dataModel;
	private JTree dataTree;
	
	private FolderNode dataSetNode;
	private FolderNode dataGroupNode;
	private FolderNode listRoot;
	
	private DefaultTreeModel listModel;
	private JTree listTree;
	
	public DataViewer (ChipMonkApplication application) {
		this.application = application;
		this.collection = application.dataCollection();
		collection.addDataChangeListener(this);
		collection.probeSet().addProbeSetChangeListener(this);
		setLayout(new GridBagLayout());
		setBackground(Color.WHITE);
		GridBagConstraints con = new GridBagConstraints();
		con.gridx=0;
		con.gridy=0;
		con.weightx=0.1;
		con.weighty=0.01;
		con.fill=GridBagConstraints.HORIZONTAL;
		con.anchor = GridBagConstraints.FIRST_LINE_START;
		
		DefaultMutableTreeNode root = new FolderNode("Data");
		dataModel = new DefaultTreeModel(root);
		
		dataSetNode = new FolderNode("Data Sets");
		root.add(dataSetNode);
		
		DataSet [] sets = collection.getAllDataSets();
		for (int i=0;i<sets.length;i++) {
			insertNodeAlphabetically(new DefaultMutableTreeNode(sets[i]), dataSetNode, dataModel);
		}
		
		
		dataGroupNode = new FolderNode("Data Groups");
		root.add(dataGroupNode);
		
		DataGroup [] groups = collection.getAllDataGroups();
		for (int i=0;i<groups.length;i++) {
			insertNodeAlphabetically(new DefaultMutableTreeNode(groups[i]), dataGroupNode, dataModel);
		}


		dataTree = new UnfocusableTree(dataModel);
		dataTree.addMouseListener(this);
		dataTree.addTreeSelectionListener(this);
		dataTree.setCellRenderer(new DataTreeRenderer());
		add(dataTree,con);
		
		con.gridy++;
		listRoot = new FolderNode("Probe Lists");
		listModel = new DefaultTreeModel(listRoot);

		ProbeList [] lists = collection.probeSet().getAllProbeLists();
		for (int i=0;i<lists.length;i++) {
			insertNodeAlphabetically(new DefaultMutableTreeNode(lists[i]), listRoot, listModel);
		}

		listTree = new UnfocusableTree(listModel);
		listTree.addTreeSelectionListener(this);
		listTree.addMouseListener(this);
		listTree.setCellRenderer(new DataTreeRenderer());
		
		
		add(listTree,con);
		
		
		// This nasty bit just makes the trees squash up to the top of the display
		// area.
		con.gridy++;
		con.weighty=1;
		con.fill = GridBagConstraints.BOTH;
		add(new JLabel(" "),con);
		
	}
	
	public DataStore [] getSelectedDataStores () {
		Vector stores = new Vector();
		TreePath [] paths = dataTree.getSelectionPaths();
		if (paths != null) {
			for (int i=0;i<paths.length;i++) {
				Object o = ((DefaultMutableTreeNode)(paths[i].getLastPathComponent())).getUserObject();
				if (o instanceof DataStore) {
					stores.add(o);
				}
			}
		}
		
		return (DataStore [])stores.toArray(new DataStore[0]);
	}
	
	public DataSet [] getSelectedDataSets () {
		TreePath [] paths = dataTree.getSelectionPaths();
		Vector selections = new Vector();
		for (int i=0;i<paths.length;i++) {
			if (((DefaultMutableTreeNode)paths[i].getLastPathComponent()).getUserObject() instanceof DataSet) {
				selections.add(((DefaultMutableTreeNode)paths[i].getLastPathComponent()).getUserObject());
			}
		}
		return (DataSet [])selections.toArray(new DataSet[0]);
	}
	
	public DataGroup [] getSelectedDataGroups () {
		TreePath [] paths = dataTree.getSelectionPaths();
		Vector selections = new Vector();
		for (int i=0;i<paths.length;i++) {
			if (((DefaultMutableTreeNode)paths[i].getLastPathComponent()).getUserObject() instanceof DataGroup) {
				selections.add(((DefaultMutableTreeNode)paths[i].getLastPathComponent()).getUserObject());
			}
		}
		return (DataGroup [])selections.toArray(new DataGroup[0]);
	}

	public void mouseClicked(MouseEvent arg0) {
	}

	public void mousePressed(MouseEvent me) {
		if ((me.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK) {
			
			if (me.getSource() == dataTree) {
				dataTree.setSelectionRow(dataTree.getRowForLocation(me.getX(),me.getY()));
				
				if (dataTree.getSelectionPath() == null) return;
				
				// The following check just ensures that they
				// don't get a menu up if they click on the root node.
				if (((DefaultMutableTreeNode)dataTree.getSelectionPath().getLastPathComponent()).getUserObject() instanceof DataSet) {
					(new DataPopupMenu((DataSet)((DefaultMutableTreeNode)dataTree.getSelectionPath().getLastPathComponent()).getUserObject())).show(dataTree,me.getX(),me.getY());
				}
				else if (((DefaultMutableTreeNode)dataTree.getSelectionPath().getLastPathComponent()).getUserObject() instanceof DataGroup) {
					(new GroupPopupMenu((DataGroup)((DefaultMutableTreeNode)dataTree.getSelectionPath().getLastPathComponent()).getUserObject())).show(dataTree,me.getX(),me.getY());
				}
			}
			else if (me.getSource() == listTree) {
				listTree.setSelectionRow(listTree.getRowForLocation(me.getX(),me.getY()));
				
				if (listTree.getSelectionPath() == null) return;
				
				// The following check just ensures that they
				// don't get a menu up if they click on the root node.
				if (((DefaultMutableTreeNode)listTree.getSelectionPath().getLastPathComponent()).getUserObject() instanceof ProbeList) {
					(new ProbePopupMenu((ProbeList)((DefaultMutableTreeNode)listTree.getSelectionPath().getLastPathComponent()).getUserObject())).show(listTree,me.getX(),me.getY());
				}
				
			}
		}
		else if (me.getClickCount() >= 2) {
			// They've double clicked on something, we'd better do something appropriate
			if (me.getSource() == listTree) {
				
				if (listTree.getSelectionPath() == null) return;

				// The following check just ensures that they
				// don't get a menu up if they click on the root node.
				if (((DefaultMutableTreeNode)listTree.getSelectionPath().getLastPathComponent()).getUserObject() instanceof ProbeList) {
					application.showProbeListDetails((ProbeList)((DefaultMutableTreeNode)listTree.getSelectionPath().getLastPathComponent()).getUserObject());
				}
			}
		}
	}

	public void mouseReleased(MouseEvent arg0) {
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseExited(MouseEvent arg0) {
	}
	
	public void valueChanged(TreeSelectionEvent tse) {
		// Check for a new selected node and act appropriately
		if (tse.getSource() == listTree) {
			if (listTree.getSelectionPath() != null && ((DefaultMutableTreeNode)listTree.getSelectionPath().getLastPathComponent()).getUserObject() instanceof ProbeList) {
				try {
					application.dataCollection().probeSet().setActiveList((ProbeList)((DefaultMutableTreeNode)listTree.getSelectionPath().getLastPathComponent()).getUserObject());
				} catch (ChipMonkException e) {
					e.printStackTrace();
				}
			}
			else {
				try {
					application.dataCollection().probeSet().setActiveList(null);
				} catch (ChipMonkException e) {
					e.printStackTrace();
				}
			}
			application.chromosomeViewer().tracksUpdated();

		}
		else if (tse.getSource() == dataTree) {
			if (dataTree.getSelectionPath() != null && ((DefaultMutableTreeNode)dataTree.getSelectionPath().getLastPathComponent()).getUserObject() instanceof DataStore) {
				try {
					application.dataCollection().setActiveDataStore((DataStore)((DefaultMutableTreeNode)dataTree.getSelectionPath().getLastPathComponent()).getUserObject());
				} catch (ChipMonkException e) {
					e.printStackTrace();
				}
			}
			else {
				try {
					application.dataCollection().setActiveDataStore(null);
				} catch (ChipMonkException e) {
					e.printStackTrace();
				}
			}
			application.chromosomeViewer().tracksUpdated();

		}
		
	}
	
	public String getNewName (String initialName) {
		String name=null;
		while (true) {
			name = (String)JOptionPane.showInputDialog(this,"Enter new name","Rename",JOptionPane.QUESTION_MESSAGE,null,null,initialName);
			if (name == null)
				return null;  // They cancelled
				
			if (name.length() == 0)
				continue; // Try again
			
			break;
		}
		if (name.equals(initialName)) {
			return null;
		}
		return name;
	}
	
	private class DataPopupMenu extends JPopupMenu implements ActionListener {

		private DataSet d;

		public DataPopupMenu (DataSet d) {
			this.d = d;
			
			JCheckBoxMenuItem displayTrack = new JCheckBoxMenuItem("Show track in chromosome view");
			displayTrack.setActionCommand("display_track");
			displayTrack.addActionListener(this);
			if (application.dataStoreIsDrawn(d)) {
				displayTrack.setState(true);
			}
			else {
				displayTrack.setState(false);
			}
			add(displayTrack);
			
			JMenuItem scatter = new JMenuItem("Show Scatter Plot");
			scatter.setActionCommand("scatter");
			scatter.addActionListener(this);
			add(scatter);

			JMenuItem ma = new JMenuItem("Show MA Plot");
			ma.setActionCommand("ma");
			ma.addActionListener(this);
			add(ma);
			
			JMenuItem rename = new JMenuItem("Rename");
			rename.setActionCommand("rename");
			rename.addActionListener(this);
			add(rename);
			

			// I'm not sure at the moment whether I should allow
			// deletion of a data set.  There are lots of places
			// this would affect - I'm not going to implement this
			// at the moment.
//			JMenuItem delete = new JMenuItem("Delete");
//			delete.setActionCommand("delete");
//			delete.addActionListener(this);
//			add(delete);


		}

		public void actionPerformed(ActionEvent ae) {
			if (ae.getActionCommand().equals("ma")) {
				application.showMAPlot(d);
			}
			else if (ae.getActionCommand().equals("scatter")) {
				application.showScatterPlot(d);
			}
			else if (ae.getActionCommand().equals("display_track")) {
				if (((JCheckBoxMenuItem)ae.getSource()).getState()) {
					application.addToDrawnDataStores(d);					
				}
				else {
					application.removeFromDrawnDataStores(d);					
				}
			}
			else if (ae.getActionCommand().equals("rename")) {
				String name = getNewName(d.name());
				if (name != null) {
					d.setName(name);
//					dataModel.nodeChanged((DefaultMutableTreeNode)dataTree.getSelectionPath().getLastPathComponent());
				}
			}
		}
	}

	private class GroupPopupMenu extends JPopupMenu implements ActionListener{
		
		private DataGroup d;
		public GroupPopupMenu (DataGroup d) {
			this.d = d;
			JCheckBoxMenuItem displayTrack = new JCheckBoxMenuItem("Show track in chromosome view");
			displayTrack.setActionCommand("display_track");
			displayTrack.addActionListener(this);
			if (application.dataStoreIsDrawn(d)) {
				displayTrack.setState(true);
			}
			else {
				displayTrack.setState(false);
			}
			add(displayTrack);
			
			JMenuItem scatter = new JMenuItem("Show Scatter Plot");
			scatter.setActionCommand("scatter");
			scatter.addActionListener(this);
			add(scatter);

			JMenuItem ma = new JMenuItem("Show MA Plot");
			ma.setActionCommand("ma");
			ma.addActionListener(this);
			add(ma);


			JMenuItem rename = new JMenuItem("Rename");
			rename.setActionCommand("rename");
			rename.addActionListener(this);
			add(rename);
			
			JMenuItem delete = new JMenuItem("Delete");
			delete.setActionCommand("delete");
			delete.addActionListener(this);
			add(delete);

		}
		
		public void actionPerformed(ActionEvent ae) {
			if (ae.getActionCommand().equals("ma")) {
				application.showMAPlot(d);
			}
			else if (ae.getActionCommand().equals("scatter")) {
				application.showScatterPlot(d);
			}

			else if (ae.getActionCommand().equals("display_track")) {
				if (((JCheckBoxMenuItem)ae.getSource()).getState()) {
					application.addToDrawnDataStores(d);					
				}
				else {
					application.removeFromDrawnDataStores(d);					
				}
			}
			else if (ae.getActionCommand().equals("rename")) {
				String name = getNewName(d.name());
				if (name != null) {
					d.setName(name);
//					dataModel.nodeChanged((DefaultMutableTreeNode)dataTree.getSelectionPath().getLastPathComponent());
				}
			}
			else if (ae.getActionCommand().equals("delete")) {
				collection.removeDataGroup(d);
				application.removeFromDrawnDataStores(d);
			}
		}
	}
	
	private class ProbePopupMenu extends JPopupMenu implements ActionListener{
		
		private ProbeList p;
		public ProbePopupMenu (ProbeList p) {
			this.p = p;
			
			JMenuItem view = new JMenuItem("View List");
			view.setActionCommand("view");
			view.addActionListener(this);
			add(view);
			
			JMenuItem rename = new JMenuItem("Rename");
			rename.setActionCommand("rename");
			rename.addActionListener(this);
			add(rename);
			
			JMenuItem delete = new JMenuItem("Delete");
			delete.setActionCommand("delete");
			delete.addActionListener(this);
			add(delete);
		}
		
		public void actionPerformed(ActionEvent ae) {
			if (ae.getActionCommand().equals("rename")) {
				String name = getNewName(p.getListName());
				if (name != null) {
					p.setName(name);
//					listModel.nodeChanged((DefaultMutableTreeNode)listTree.getSelectionPath().getLastPathComponent());
				}
			}
			else if (ae.getActionCommand().equals("delete")) {
				collection.probeSet().removeProbeList(p);
			}
			
			else if (ae.getActionCommand().equals("view")) {
				application.showProbeListDetails(p);
			}
		}
	}
	
	private class FolderNode extends DefaultMutableTreeNode {
		public FolderNode (String s) {
			super(s);
		}
		public boolean isLeaf () {
			return false;
		}
	}
	
	
	private class UnfocusableTree extends JTree {

		// This class is needed to make sure the arrow key navigation
		// always works in the chromosome view.  If either of the JTrees
		// can grab focus they will intercept the arrow key events and
		// just move the selections on the tree.
		
		public UnfocusableTree (TreeModel m) {
			super(m);
			this.setFocusable(false);
		}
				
	}

	// Methods from DataChangeListener
	
	public void dataGroupAdded(DataGroup g) {
		DefaultMutableTreeNode n = new DefaultMutableTreeNode(g);
		insertNodeAlphabetically(n, dataGroupNode, dataModel);
//		dataGroupNode.add(n);
//		dataModel.nodesWereInserted(dataGroupNode,new int [] {dataGroupNode.getIndex(n)});
	}

	public void dataGroupRemoved(DataGroup g) {
		Enumeration e = dataGroupNode.children();
		while (e.hasMoreElements()) {
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)e.nextElement();
			if (n.getUserObject() == g) {
				int i = dataGroupNode.getIndex(n);
				dataGroupNode.remove(i);
				dataModel.nodesWereRemoved(dataGroupNode,new int [] {i},new Object [] {n});
			}
		}
	}

	public void dataSetAdded(DataSet d) {
		DefaultMutableTreeNode n = new DefaultMutableTreeNode(d);
		insertNodeAlphabetically(n, dataSetNode, dataModel);
	}

	public void dataSetRemoved(DataSet d) {
		Enumeration e = dataSetNode.children();
		while (e.hasMoreElements()) {
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)e.nextElement();
			if (n.getUserObject() == d) {
				int i = dataSetNode.getIndex(n);
				dataSetNode.remove(i);
				dataModel.nodesWereRemoved(dataSetNode,new int [] {i},new Object [] {n});
			}
		}
	}

	// Methods from ProbeSetChangeListener
	
	public void probeListAdded(ProbeList p) {
		DefaultMutableTreeNode n = new DefaultMutableTreeNode(p);
		insertNodeAlphabetically(n, listRoot, listModel);
	}

	public void probeListRemoved(ProbeList p) {
		Enumeration e = listRoot.children();
		while (e.hasMoreElements()) {
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)e.nextElement();
			if (n.getUserObject() == p) {
				int i = listRoot.getIndex(n);
				listRoot.remove(i);
				listModel.nodesWereRemoved(listRoot,new int [] {i},new Object [] {n});
			}
		}
	}

	public void dataGroupRenamed(DataGroup g) {

		// The easiest thing to do is to remove and re-add
		// the node.  We need to check that it's present in
		// the tree first to avoid duplication.
	
		Enumeration e = dataGroupNode.children();
		while (e.hasMoreElements()) {
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)e.nextElement();
			if (n.getUserObject() == g) {
				int i = dataGroupNode.getIndex(n);
				dataGroupNode.remove(i);
				dataModel.nodesWereRemoved(dataGroupNode,new int [] {i},new Object [] {n});
				dataGroupAdded(g);
			}
		}

	}

	public void dataGroupSamplesChanged(DataGroup g) {
	}

	public void dataSetRenamed(DataSet d) {

		// The easiest thing to do is to remove and re-add
		// the node.  We need to check that it's present in
		// the tree first to avoid duplication.
		
		Enumeration e = dataSetNode.children();
		while (e.hasMoreElements()) {
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)e.nextElement();
			if (n.getUserObject() == d) {
				int i = dataSetNode.getIndex(n);
				dataSetNode.remove(i);
				dataModel.nodesWereRemoved(dataSetNode,new int [] {i},new Object [] {n});
				dataSetAdded(d);
			}
		}
	}

	public void probeListRenamed(ProbeList l) {

		// The easiest thing to do is to remove and re-add
		// the node.  We need to check that it's present in
		// the tree first to avoid duplication.
		
		Enumeration e = listRoot.children();
		while (e.hasMoreElements()) {
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)e.nextElement();
			if (n.getUserObject() == l) {
				int i = listRoot.getIndex(n);
				listRoot.remove(i);
				listModel.nodesWereRemoved(listRoot,new int [] {i},new Object [] {n});
				probeListAdded(l);
			}
		}

	}
	
	private void insertNodeAlphabetically (MutableTreeNode child, MutableTreeNode parent, DefaultTreeModel model) {
		// Go through the existing children to find the
		// point at which to insert.  Do this in a case
		// insensitive way
		int index = 0;
		Enumeration e = parent.children();
		while (e.hasMoreElements()) {
			if (e.nextElement().toString().toLowerCase().compareTo(child.toString().toLowerCase()) > 0) {
				break;
			}
			index++;
		}
		model.insertNodeInto(child, parent, index);
	}

}
